#------------------------------------------------------------------
#  RADWAV - 08|14|2014
#  This file performs multiprocessing using pyaudio
#------------------------------------------------------------------
from multiprocessing import Process, Queue
import time
import pyaudio
import numpy as np
from scipy import signal
import subprocess

CHUNK = 8192#24000
RATE = 48000
WIDTH = 2
CHANNELS = 1

TWOPI = 6.2832

N = 400 #400

def captureData(qIn, streamIn): 
#--------------------------------------------------------------------------
# captureData(qIn, streamIn) collects data from the pyAudio streamIn 
# device and places this data into the Queue 'qIn' where the data awaits
# processing.
# RADWAV.COM - 08|16|2014
#--------------------------------------------------------------------------
  print "Starting Collection."
  t0 = time.time()
  #for i in range(N):
  while(True):
    data = streamIn.read(CHUNK)
    print "Capture Waited:"
    print (time.time() - t0)
    t0 = time.time()
    qIn.put(data)
    
  print "Collection Finished"

def processData(qIn, qOut): 
#---------------------------------------------------------------------------
# processData(qIn, qOut) processes data from the Queue 'qIn' and places
# the result into the Queue 'qOut'
# RADWAV.COM - 08|16|2014
#---------------------------------------------------------------------------
  print "Starting Processing."

  #processing parameters for band-pass filter
  #num = np.array([1.0, 0.0, -1.0])
  #den = np.array([1, 0.1122, 0.905])
  num = np.array([1.0, 0.0, -2.0, 0.0, 1.0])
  den = np.array([1.0, -0.7894, 1.6829, -0.6184, 0.6181])
 

  zf = signal.lfilter_zi(num, den)
  axis = -1

  #processing parameters for digital down-conversion
  freq = 12000.0  #must be int number of cycles per CHUNK
  complexLo = np.exp(-1j * TWOPI * freq / RATE * np.linspace(1, CHUNK, CHUNK))

  #init agc values
  gain = 1
  max_gain = 10000
  decay = 1.4
  peak = 16384

  i = 0
  #for i in range(N):  
  while(True):
    t0 = time.time()
    #time.sleep(1.0)
    data = qIn.get()
    print "Processing Waited %f" % (time.time() - t0)
    #print data    
    npDataIn = np.fromstring(data, 'Int16')
    if(i==0):
      npDataOut = npDataIn
      i=1  

    npDataFilt, zf = signal.lfilter(num, den, npDataIn, axis, zf)
    npDownConvCmplx = npDataFilt * complexLo
    npDataOut = (npDownConvCmplx.real)

    #Digital Automatic Gain Control--------------------------------------------
    maxVal = npDataOut.max()
    gain = min(gain * decay, max_gain)
    if((gain * maxVal) > peak):
      gain = peak / maxVal  #drop gain when there is a sudden increase

    npDataOut = npDataOut * gain
    print "Gain: ", gain, "Max Val: ", maxVal
    #print "Max Val: ",maxVal
    

    #npDataOut = npDataIn  #all-pass filter
    strData = npDataOut.astype(np.int16).tostring()
   
    qOut.put(strData)
    print "Queue Length In:"
    print qIn.qsize()
    print "Queue Length Out:"
    print qOut.qsize()

def playData(q, streamOut):
#---------------------------------------------------------------------------
#  playData(q, streamOut) plays sound data from the Queue 'q' using the 
#  pyAudio device connected to streamOut
#  RADWAV.COM - 08|16|2014
#---------------------------------------------------------------------------
  print "Starting Audio Output Buffer."
  #Buffer Data before starting
  minBuffer = 4  #in unit of [Chunks]
  while(q.qsize() < minBuffer):
    time.sleep(0.1)
  #for i in range(N):
  while(True): 
    data = q.get()
    streamOut.write(data, CHUNK)

#---------------------------------------------------------------------------
#  listAudioDevices(p) shows which audio hardware is available
#  RADWAV.COM - 08|16|2014
#---------------------------------------------------------------------------
def listAudioDevices(p):
  count = p.get_device_count()
  devices = []
  for i in range(count):
    devices.append(p.get_device_info_by_index(i))
  
  for i, dev in enumerate(devices):
    print "%d - %s" % (i, dev['name'])
    if 'C-Media USB' in dev['name']:
      print "[INFO] - Found C-Media USB as device %i" % i
      selectDeviceNumber = i

  try:
    selectDeviceNumber
  except NameError:
    print "[WARNING] - C-Media USB Sound Device Not Found."
  else:
    print "[INFO] - Using C-Media USB Sound Device"

  return selectDeviceNumber

#--------------------------------------------------------------------
#  This function is used to start the receiver processing
#--------------------------------------------------------------------
def startProcessing():
  #force alsa to reload
  commandString = "sudo alsa force-reload"
  print commandString
  subprocess.call(commandString, shell=True)

  p = pyaudio.PyAudio() 
  selectDeviceNumber = 0
  selectDeviceNumber = listAudioDevices(p)
  
  #input_device_index = int(raw_input('Choose source: '))
  #output_device_index = int(raw_input('Choose output: '))

  print "[INFO] - Device %i is selected as the input and output sound card." % selectDeviceNumber

  input_device_index = selectDeviceNumber
  output_device_index = selectDeviceNumber

  streamIn = p.open(format=p.get_format_from_width(WIDTH),
                    channels = CHANNELS,
                    rate = RATE,
                    input = True,
                    input_device_index = input_device_index,
                    frames_per_buffer = CHUNK)

  streamOut = p.open(format=p.get_format_from_width(WIDTH),
                    channels = CHANNELS,
                    rate = RATE,
                    output = True,
                    output_device_index = output_device_index,
                    frames_per_buffer = CHUNK)

   
  qIn = Queue()  #buffer between input and processing
  qOut = Queue() #buffer between processing and output

  #connect the blocks
  capture = Process(target = captureData, args=(qIn,streamIn))
  processing = Process(target = processData, args=(qIn,qOut))
  play = Process(target = playData, args=(qOut, streamOut))
  
  #start the blocks
  capture.start()
  processing.start()
  play.start()
  print "Waiting..........."
  capture.join()
  processing.join()
  play.join()

  print "Finishing......"

  #clean-up the streams
  streamIn.stop_stream()
  streamOut.stop_stream()
  streamIn.close()
  streamOut.close()
  p.terminate()


#----------------------------------------------------------------------------
#  main program begins here
if __name__ == '__main__':

  startReceiver()

else:
  print "[STATUS] - Imported radwavRxProc.py"
  

