#-------------------------------------------------------------------------------
#  Title:   radwavSendMorse.py
#  Version: 0.1pm - FEB 16, 2015.
#  
#  Description:  radwavSendMorse.py sends Morse code from the RadWAV SkyPi-40.  
#  Details about the hardware are available on the web:
#    http://www.radwav.com
#
#  Copyright 2015 RADWAV - Scottsdale, AZ, USA
#
#  License:  
#
#    This file is part of radwavSkyPi.
#
#    radwavSkyPi is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    radwavSkyPi is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with radwavSkyPi.  If not, see <http://www.gnu.org/licenses/>.
#
#-------------------------------------------------------------------------------

import RPi.GPIO as GPIO
import time

def dot(speed):
  #send a Morse code dot
  GPIO.output(keyPin, GPIO.HIGH)      #turn buffer amplifier on
  time.sleep(1.0/speed)               #sleep during "dit"
  GPIO.output(keyPin, GPIO.LOW)       #turn power amplifier off
  time.sleep(1.0/speed)

def dash(speed):
  #send a Morse code dit
  GPIO.output(keyPin, GPIO.HIGH)      #turn buffer amplifier on
  time.sleep(3.0/speed)               #sleep during "dah"
  GPIO.output(keyPin, GPIO.LOW)       #turn power amplifier off
  time.sleep(1.0/speed)

def wordSpace(speed):
  #print "word space"
  time.sleep(6.0/speed)

def txMorseMessage(sendText, speed):
  print "[STATUS] - Called radwavSendMorse.py with: %s at speed: %i" % (sendText, speed)

  GPIO.setmode(GPIO.BOARD)            #numbering follows pin numbers on RPi header P1
  GPIO.setup(txEnablePin, GPIO.OUT)   #set pins to output status
  GPIO.setup(keyPin, GPIO.OUT)

  GPIO.output(txEnablePin, GPIO.LOW)  #turn buffer amplifier off
  GPIO.output(keyPin, GPIO.LOW)       #turn power amplifier off

  #convert the text to transmitted Morse code
  for n in sendText:
    GPIO.output(txEnablePin, GPIO.HIGH)  #turn buffer amplifier off
    lookUpTable(n, speed)
    GPIO.output(txEnablePin, GPIO.LOW)   #turn buffer amplifier off
    GPIO.output(keyPin, GPIO.LOW)        #turn buffer amplifier off

  GPIO.cleanup()

def lookUpTable(n, speed):
  if n == 'A':
    dot(speed)
    dash(speed)
  elif n == 'B':
    dash(speed)
    dot(speed)
    dot(speed)
    dot(speed)
  elif n == 'C':
    dash(speed)
    dot(speed)
    dash(speed)
    dot(speed)
  elif n == 'D':
    dash(speed)
    dot(speed)
    dot(speed)
  elif n == 'E':
    dot(speed)
  elif n == 'F':
    dot(speed)
    dot(speed)
    dash(speed)
    dot(speed)
  elif n == 'G':
    dash(speed)
    dash(speed)
    dot(speed)
  elif n == 'H':
    dot(speed)
    dot(speed)
    dot(speed)
    dot(speed)
  elif n == 'I':
    dot(speed)
    dot(speed)
  elif n == 'J':
    dot(speed)
    dash(speed)
    dash(speed)
    dash(speed)
  elif n == 'K':
    dash(speed)
    dot(speed)
    dash(speed)
  elif n == 'L':
    dot(speed)
    dash(speed)
    dot(speed)
    dot(speed)
  elif n == 'M':
    dash(speed)
    dash(speed)
  elif n == 'N':
    dash(speed)
    dot(speed)
  elif n == 'O':
    dash(speed)
    dash(speed)
    dash(speed)
  elif n == 'P':
    dot(speed)
    dash(speed) 
    dash(speed)
    dot(speed)
  elif n == 'Q':
    dash(speed)
    dash(speed)
    dot(speed)
    dash(speed)
  elif n == 'R':
    dot(speed)
    dash(speed)
    dot(speed)
  elif n == 'S':
    dot(speed)
    dot(speed)
    dot(speed)
  elif n == 'T':
    dash(speed)
  elif n == 'U':
    dot(speed)
    dot(speed)
    dash(speed)
  elif n == 'V':
    dot(speed)
    dot(speed)
    dot(speed)
    dash(speed)
  elif n == 'W':
    dot(speed)
    dash(speed)
    dash(speed)
  elif n == 'X':
    dash(speed)
    dot(speed)
    dot(speed)
    dash(speed)
  elif n == 'Y':
    dash(speed)
    dot(speed)
    dash(speed)
    dash(speed)
  elif n == 'Z':
    dash(speed)
    dash(speed)
    dot(speed)
    dot(speed)
  elif n == '0':
    dash(speed)
    dash(speed) 
    dash(speed)
    dash(speed)
    dash(speed)
  elif n == '1':
    dot(speed)
    dash(speed) 
    dash(speed)
    dash(speed)
    dash(speed)
  elif n == '2':
    dot(speed)
    dot(speed) 
    dash(speed)
    dash(speed)
    dash(speed)
  elif n == '3':
    dot(speed)
    dot(speed) 
    dot(speed)
    dash(speed)
    dash(speed)
  elif n == '4': 
    dot(speed)
    dot(speed) 
    dot(speed)
    dot(speed)
    dash(speed)
  elif n == '5': 
    dot(speed)
    dot(speed) 
    dot(speed)
    dot(speed)
    dot(speed)
  elif n == '6':
    dash(speed)
    dot(speed) 
    dot(speed)
    dot(speed)
    dot(speed)
  elif n == '7':
    dash(speed)
    dash(speed) 
    dot(speed)
    dot(speed)
    dot(speed)
  elif n == '8':
    dash(speed)
    dash(speed) 
    dash(speed)
    dot(speed)
    dot(speed) 
  elif n == '9':
    dash(speed)
    dash(speed) 
    dash(speed)
    dash(speed)
    dot(speed)
  elif n == '/':
    dash(speed)
    dot(speed)
    dot(speed)
    dash(speed)
    dot(speed)
  elif n == '?':
    dot(speed)
    dot(speed)
    dash(speed)
    dash(speed)
    dot(speed)
    dot(speed)
  elif n == ',':
    dash(speed)
    dot(speed)
    dash(speed)
    dash(speed) 
  elif n == '.':
    dot(speed)
    dash(speed)
    dot(speed)
    dash(speed)
    dot(speed)
    dash(speed)
  elif n == ' ':
    wordSpace(speed)
  else: 
    print "[WARNING] - radwavSendMorse:  character not in list"

  #print "char space"
  time.sleep(4.0/speed)  #character space

#main routine is here-----------------------------------------------------------
txEnablePin = 16                    #buffer amplifier enabled if "true"
keyPin = 18                         #power amplifier enable if set to GPIO.HIGH


if __name__ == '__main__':  #this part executes if this file is launched directly
  sendText = "CQ CQ CQ DE RADWAV K"           #message for testing the routine
  speed = 10                     #the dit lasts 1/speed in seconds
  txMorseMessage(sendText, speed)
else:                       #this part executes is another routine called this
  print "[STATUS] - Imported radwavSendMorse.py"

